import logging
import urllib.parse
import tempfile
from pathlib import Path
from contextlib import contextmanager

import requests

logger = logging.getLogger(__name__)

@contextmanager
@staticmethod
def download(uri: str, session: requests.Session):

    logger.info(f'Downloading "{uri}"...')
    with session.get(uri) as response:
        response.raise_for_status()
        yield response
        

@contextmanager
def download_temp_file(uri: str, session: requests.Session):   

    # Get filename from URL
    filename = Path(urllib.parse.urlsplit(uri).path).name

    with download(uri, session=session) as response:
        # Store file on local disk
        with tempfile.TemporaryDirectory() as directory:
            with Path(directory).joinpath(filename).open('wb') as file:
                file.write(response.content)
                yield file



#!/usr/bin/env python3

import argparse
import logging
import datetime
import itertools
from typing import Optional

from pathlib import Path

import dateutil.parser
import pytz
import requests

from .twitter import read_twitter_json, simplify_tweet
from .pdf import PDFDocument
from .settings import ENCODING, LANGUAGE, FONT_FAMILY, FONT_SIZE, FONT, IMAGE_WIDTH

DESCRIPTION = """
Parse Twitter archived JSON files and convert to PDF files
"""


logger = logging.getLogger(__name__)


def utc_timestamp(timestamp: str) -> datetime.datetime:
    """
    Parse user-inputted timestamp
    """
    # Parse timestamp
    t = dateutil.parser.parse(timestamp)
    # Convert to UTC
    return pytz.UTC.localize(t)


def get_args():
    """
    Command-line arguments
    """

    parser = argparse.ArgumentParser(description=DESCRIPTION)

    # Twitter archive options
    parser.add_argument("-f", "--file", type=Path, dest="filename", help="Path to Twitter JSON archive", required=True)
    parser.add_argument('--encoding', default=ENCODING, help='Input file character set')
    parser.add_argument("-a", "--hashtags", help="Filter by hashtags (comma-separated list)", default=str())
    parser.add_argument("-s", "--date_start", dest="date_start", type=utc_timestamp)
    parser.add_argument("-e", "--date_end", dest="date_end", type=utc_timestamp)
    parser.add_argument('--start', type=int, help="Start at this tweet number")
    parser.add_argument('--stop', type=int, help="Stop after this many tweets")

    # PDF output options
    parser.add_argument("-p", "--pdf", type=Path, help='Output PDF file path', required=True)
    parser.add_argument('-i', '--images', help='Download images and put in the PDF document (SLOW)', action='store_true')
    parser.add_argument('-w', '--imgwidth', help="Image width", default=IMAGE_WIDTH)
    
    # Font options
    parser.add_argument('--font', help='Unicode font TTF file', type=Path, default=FONT)
    parser.add_argument('--font_family', default=FONT_FAMILY)
    parser.add_argument('--font_size', type=float, default=FONT_SIZE)
    parser.add_argument('--loglevel', default='INFO', help="Verbosity level: DEBUG, INFO, WARNING, ERROR")
    parser.add_argument('--language', default=LANGUAGE, help='Language ISO 639-1 code')

    return parser.parse_args()


def main():
    options = get_args()
    logging.basicConfig(level=options.loglevel)

    # Get the selected hashtags
    hashtag_filter = {s.strip() for s in options.hashtags.casefold().split(',')}

    pdf = PDFDocument(font=options.font, font_family=options.font_family, font_size=options.font_size)

    session: Optional[requests.Session] = None
    if options.images:
        session = requests.Session()

    # Iterate over input tweets
    tweet_count = 0
    for i, tweet in itertools.islice(enumerate(read_twitter_json(options.filename, encoding=options.encoding)), options.start, options.stop):
        tweet_count += 1

        # Show progress
        if tweet_count % 1000 == 0:
            logger.info(f"Processing tweet {tweet_count}")
        logger.debug(tweet)

        # Decode tweet in a simple structure
        tweet_simple = simplify_tweet(tweet, language=options.language)

        # Time filter
        try:
            if tweet_simple['created_at'] < options.date_start:
                continue
        except TypeError:
            pass
        try:
            if tweet_simple['created_at'] >= options.date_end:
                continue
        except TypeError:
            pass

        # Filter by hashtag
        if not tweet_simple['hashtags'].union(hashtag_filter):
            continue

        pdf.add_tweet(tweet_simple, download_images=options.images, image_width=options.imgwidth, session=session)

    pdf.output(options.pdf)


if __name__ == '__main__':
    main()



import json
from pathlib import Path
from typing import Iterator
from collections.abc import Mapping, Sequence
import urllib.parse
from http import HTTPStatus
import logging

import dateutil.parser
import emoji

from .settings import ENCODING, LANGUAGE

logger = logging.getLogger(__name__)


def read_twitter_json(path: Path, encoding=None) -> Sequence[Mapping]:
    encoding = encoding or ENCODING

    # Load input file into memory
    with path.open(encoding=encoding) as file:
        data = file.read()

    # Remove everything before the first square bracket
    data = '[' + data.partition('[')[2]

    # Parse JSON
    return [doc['tweet'] for doc in json.loads(data)]


def remove_emoji(s: str, lang: str = None) -> str:
    """
    Remove emojis, which could break PDF output

    https://carpedm20.github.io/emoji/docs/#replacing-and-removing-emoji
    """

    def replace(s, data_dict):
        nonlocal lang

        try:
            return data_dict[lang]
        # If this language isn't there, default
        except KeyError:
            return data_dict[LANGUAGE]

    # Replace emoji symbols with text representation
    return emoji.replace_emoji(s, replace=replace)

def simplify_tweet(tweet: Mapping, username: str = None, language: str = None) -> Mapping:
    """
    Gets data from tweet and returns a simplified data structure

    :returns: Simplified object
    """

    # Language code ISO 639-1
    lang = tweet.get('lang', language or LANGUAGE)

    # Remove undetermined language (any language match) or artificial language
    if lang in {'und', 'art'}:
        lang = None

    # Map Twitter short-URLs to full URLs
    urls = {url['url']: url['expanded_url'] for url in tweet['entities']['urls']}
    images = set()

    # Get media URLs (these also map to short urls) and add these to the short-URL map
    try:
        for media in tweet['extended_entities']['media']:
            urls[media['url']] = media['media_url']
            images.add(media['media_url'])
    except KeyError:
        pass

    # Process message body
    # Remove emoji characters (these don't convert to PDF very well)
    full_text = remove_emoji(tweet['full_text'], lang=lang)

    # Remove crazy characters, which will also break PDF
    # https://docs.python.org/3/howto/unicode.html
    try:
        full_text = full_text.encode('utf-8').decode('unicode-escape')
    except UnicodeDecodeError as exc:
        logger.error(exc)
        logger.warning(full_text)
        pass

    # Convert short urls (t.co) into their original links
    for short_url, expanded_url in urls.items():
        full_text = full_text.replace(short_url, expanded_url)

    return dict(
        created_at=dateutil.parser.parse(tweet['created_at']),
        # Get the set of all hashtags
        hashtags={tag['text'] for tag in tweet['entities']['hashtags']},
        full_text=full_text,
        # Build the original URL of the tweet
        uri=f"https://twitter.com/{username or '_'}/status/{tweet['id_str']}",
        images=images,
    )



from pathlib import Path

# Input file character set
ENCODING = 'utf-8'

# Default language code
# https://en.wikipedia.org/wiki/ISO_639-1
LANGUAGE = 'en'

FONT_FAMILY = 'DejaVu Sans'
FONT_SIZE = 11
FONT = Path('./font/DejaVuSans.ttf')
UNIT = 'mm'
IMAGE_WIDTH = 150



import tempfile
import logging
from pathlib import Path
from collections.abc import Mapping
from http import HTTPStatus
import urllib.parse

import requests
import fpdf

import tweets2pdf.utils
from .settings import FONT_FAMILY, FONT_SIZE, FONT, IMAGE_WIDTH, UNIT

logger = logging.getLogger(__name__)

session = requests.Session()

class PDFDocument:
    """
    Create a PDF document using PyFPDF
    https://pyfpdf.readthedocs.io/en/latest/
    """

    def __init__(self, font: Path = None, font_family: str = None, font_size: float = None, unit:str =None, style: str=None, **kwargs):
        self.unit = unit or UNIT
        self.pdf = fpdf.FPDF(unit=self.unit, **kwargs)
        self.font = Path(font or FONT)
        self.font_family = font_family or FONT_FAMILY
        self.font_size = font_size or FONT_SIZE
        self.style = style or ''

        # Use a Unicode font so we can use full UTF-8 character set
        # https://pyfpdf.readthedocs.io/en/latest/Unicode/index.html
        self.pdf.add_font(family=self.font_family, style=self.style, fname=self.font, uni=True)
        self.pdf.set_font(family=self.font_family, style=self.style, size=self.font_size)

        self.pdf.add_page()

    def add_tweet(self, tweet: Mapping, height: float = None, download_images: bool = False, image_width = None, session: requests.Session = None):
        """
        Append the tweet to the PDF document
        """

        image_width = image_width or IMAGE_WIDTH

        # Line height
        height = height or self.font_size * 0.6

        # Timestamp
        self.pdf.cell(w=0, h=height, txt=tweet['created_at'].isoformat(), ln=1)
        # Hyperlink to Tweet
        self.pdf.cell(w=0, h=height, txt=tweet['uri'], link=tweet['uri'], ln=1)
        # Text body
        self.pdf.multi_cell(w=0, h=height, txt=tweet['full_text'], border='B', align='L')

        if download_images:
            for image_uri in tweet['images']:
                self.add_image(image_uri, w=image_width, session=session)

    def add_image(self, image_uri: str, session: requests.Session, w = None, **kwargs):
        # Download image
        with tweets2pdf.utils.download_temp_file(image_uri, session=session) as file:
            try:
                # Insert picture into PDF document
                self.pdf.image(file.name, link=image_uri, w=w or 50, **kwargs)
            
            # Handle errors e.g. image format incompatibilities
            except RuntimeError as exc:
                logger.error(exc)
                logger.warning('Skipping image')
                # Put some text in the PDF document as a placeholder
                self.cell(image_uri, link=image_uri)

    def output(self, name, *args, **kwargs):
        logger.info('Writing PDF file...')
        self.pdf.output(name=name, *args, **kwargs)
        logger.info('Wrote PDF file "%s"', name)

    def line_break(self, height: int = 10):
        self.pdf.ln(h=height)

    def cell(self, text:str, width=0, height=None, ln: int = 1, **kwargs):
        # Line height
        height = height or self.font_size * 0.6
        self.pdf.cell(w=width, h=height, txt=text, ln=ln, **kwargs)
